@@ -36,13 +36,15 @@ import net.morilib.automata.NFAState;
 import net.morilib.automata.TextBound;
 import net.morilib.automata.dfa.ConvertedRangeDFA;
 import net.morilib.automata.dfa.DFAs;
+import net.morilib.automata.nfa.NFAObject;
 import net.morilib.automata.nfa.RegexParseException;
-import net.morilib.automata.nfa.RegexpParser;
 import net.morilib.range.CharSets;
 import net.morilib.range.Interval;
 import net.morilib.range.IntervalsInt;
 import net.morilib.range.Range;
 import net.morilib.range.RangeAdder;
+import net.morilib.regex.dfa.CharSet;
+import net.morilib.regex.dfa.RegexParser;
 import net.morilib.util.TreeSectionMap;
 import net.morilib.util.Tuple2;
 
@@ -56,11 +58,12 @@ public class DFABuilder implements NinaAction {
 	private static class W {
 
 		private CharSequence wrap;
-		private int quote;
+		private int quote, quoteFlag;
 
-		W(CharSequence s, int qc) {
+		W(CharSequence s, int qc, int qf) {
 			wrap = s;
 			quote = qc;
+			quoteFlag = qf;
 		}
 
 	}
@@ -151,6 +154,54 @@ public class DFABuilder implements NinaAction {
 
 	}
 
+	private static class WS {
+
+		private String str;
+
+		WS(String s) {
+			str = s;
+		}
+
+	}
+
+	private static class BS {
+
+		private int seq;
+
+		BS(int s) {
+			seq = s;
+		}
+
+	}
+
+	private static class FS {
+
+		private int length;
+		private String cset;
+
+		FS(int l, String s) {
+			length = l;
+			cset = s;
+		}
+
+	}
+
+	private static class FI {
+
+		private int length;
+		private String flag;
+		private Range range;
+		private int radix;
+
+		FI(int l, String f, Range r, int rd) {
+			length = l;
+			flag = f;
+			range = r;
+			radix = rd;
+		}
+
+	}
+
 	/**
 	 * 
 	 *
@@ -560,6 +611,7 @@ public class DFABuilder implements NinaAction {
 	private static final Object _ID_C = new Object();
 
 	//
+	private static final Range SPACES = CharSets.parse(" \t\r\n\b");
 	private static final Range DIGIT =
 			IntervalsInt.newClosedInterval('0', '9');
 	private static final Range SIGN_P =
@@ -587,18 +639,12 @@ public class DFABuilder implements NinaAction {
 			ID_C_START
 			.join(IntervalsInt.newClosedInterval('0', '9'));
 	private static final Range ID_JAVA_START =
-			ID_C_START
-			.join(IntervalsInt.newClosedInterval('$', '$'))
-			.join(IntervalsInt.newClosedInterval(0x80, 0xfffc));
+			CharSets.parse("\\p{JavaIdentifierStart}");
 	private static final Range ID_JAVA_PART =
-			ID_JAVA_START
-			.join(IntervalsInt.newClosedInterval('0', '9'));
-	private static final Range NOT_WORD = CharSets.complement(
-			IntervalsInt.newClosedInterval('A', 'Z')
-			.join(IntervalsInt.newClosedInterval('a', 'z'))
-			.join(IntervalsInt.newClosedInterval('_', '_'))
-			.join(IntervalsInt.newClosedInterval('0', '9'))
-			.join(IntervalsInt.newClosedInterval(0x80, 0x7ffffffe)));
+			CharSets.parse("\\p{JavaIdentifierPart}");
+	private static final Range WORD = CharSets.parse("\\p{L}");
+	private static final Range NOT_WORD = CharSets.parse("\\P{L}");
+	private static final Range MARK = CharSets.parse("\\p{M}");
 
 	//
 	private Map<String, DBS> map = new HashMap<String, DBS>();
@@ -613,6 +659,7 @@ public class DFABuilder implements NinaAction {
 	private int mealyType;
 	private Object mealy;
 	private String name;
+	private int nfaFlag;
 
 	public DBS createState(NinaEvent q) {
 		DBS s;
@@ -644,7 +691,9 @@ public class DFABuilder implements NinaAction {
 		if(vertex.identical == null) {
 			vertex.accepted = accept;
 		} else {
-			for(DBS x : vertex.identical)  x.accepted = accept;
+			for(DBS x : vertex.identical) {
+				x.accepted = accept;
+			}
 		}
 		vertex.label = wr.toString();
 	}
@@ -654,6 +703,14 @@ public class DFABuilder implements NinaAction {
 		return a;
 	}
 
+	private DBS _linkv(NinaEvent q, DBS x, DBS a, int c, String o) {
+		Range r;
+
+		r = Interval.newPoint(Integer.valueOf(c));
+		x.linkAlphabet(a, r, o);
+		return a;
+	}
+
 	private DBS _linkc(NinaEvent q, DBS x, int c, String o) {
 		Range r;
 		DBS a;
@@ -671,39 +728,126 @@ public class DFABuilder implements NinaAction {
 		return _linkv(q, x, vertex, r, o);
 	}
 
-	private void _linkdfa1(NinaEvent q, DFA<Object, NFAState, Void> d,
-			DBS v, DBS z, Object o, String lh, String lc, boolean en) {
-		Collection<DFAState<Object, NFAState, Void>> c;
-		Map<DFAState<Object, NFAState, Void>, DBS> m;
-		DFAState<Object, NFAState, Void> h;
-		String g;
+	private Range _regexAllAlphabets(
+			DFAState<Object, NFAState, Integer> s) {
+		RangeAdder r;
+
+		r = new RangeAdder();
+		for(Interval a : s.getAlphabetRanges()) {
+			r.add(a);
+		}
+		return r.toRange();
+	}
+
+	private Range _regexFirst(DFA<Object, NFAState, Integer> d) {
+		return _regexAllAlphabets(d.getInitialState());
+	}
+
+	private DBS _linkdfa0(DFA<Object, NFAState, Integer> d,
+			DFAState<Object, NFAState, Integer> y,
+			Map<DFAState<Object, NFAState, Integer>, DBS> m, DBS v,
+			String lh) {
+		DFAState<Object, NFAState, Integer> h;
 		DBS x, w;
 
+		if(y.equals(d.getInitialState())) {
+			x = v;
+		} else if((x = m.get(y)) == null) {
+			m.put(y, x = new DBS());
+		}
+
+		for(Interval a : y.getAlphabetRanges()) {
+			h = y.go(a.getInfimumBound());
+			if(h.equals(d.getInitialState())) {
+				x.linkAlphabet(v, a, lh);
+			} else {
+				if((w = m.get(h)) == null) {
+					m.put(h, w = new DBS());
+				}
+				x.linkAlphabet(w, a, lh);
+			}
+		}
+		return x;
+	}
+
+	private void _linkdfa1(NinaEvent q,
+			DFA<Object, NFAState, Integer> d, DBS v, DBS z,
+			Object o, String lh, String lc1, String lcb, String lce,
+			boolean en) {
+		Collection<DFAState<Object, NFAState, Integer>> c;
+		Map<DFAState<Object, NFAState, Integer>, DBS> m;
+		String g;
+		DBS x;
+
 		c = DFAs.allStateRanges(d.getInitialState());
-		m = new HashMap<DFAState<Object, NFAState, Void>, DBS>();
-		g = lh + lc;
+		m = new HashMap<DFAState<Object, NFAState, Integer>, DBS>();
+		g = lc1;
 		if(o != null)  g += o.toString();
-		for(DFAState<Object, NFAState, Void> y : c) {
-			if(y.equals(d.getInitialState())) {
-				x = v;
-			} else if((x = m.get(y)) == null) {
-				m.put(y, x = new DBS());
-			}
-
+		for(DFAState<Object, NFAState, Integer> y : c) {
+			x = _linkdfa0(d, y, m, v, lh);
 			if(y.isAccepted()) {
-				x.linkOthers(z, g);
-				if(en)  x.linkEnd(z, o);
+				x.linkOthers(z, lcb + g + lce);
+				if(en)  x.linkEnd(z, g);
 			}
+		}
+	}
 
-			for(Interval a : y.getAlphabetRanges()) {
-				h = y.go(a.getInfimumBound());
-				if(h.equals(d.getInitialState())) {
-					x.linkAlphabet(v, a, lh);
+	private void _linkdfaAssert(NinaEvent q,
+			DFA<Object, NFAState, Integer> d, DBS v, DBS z,
+			Object o, String lh, String lc1, String lcb, String lce,
+			boolean en) {
+		Collection<DFAState<Object, NFAState, Integer>> c;
+		Map<DFAState<Object, NFAState, Integer>, DBS> m;
+		String g;
+		Range r;
+		DBS x;
+
+		c = DFAs.allStateRanges(d.getInitialState());
+		m = new HashMap<DFAState<Object, NFAState, Integer>, DBS>();
+		g = lc1;
+		if(o != null)  g += o.toString();
+		for(DFAState<Object, NFAState, Integer> y : c) {
+			x = _linkdfa0(d, y, m, v, lh);
+			if(y.isAccepted()) {
+				r = CharSets.complement(_regexAllAlphabets(y));
+				if(y.equals(d.getInitialState())) {
+					x.linkAlphabet(z, r,
+							q.getLanguageOption().getUnget());
 				} else {
-					if((w = m.get(h)) == null) {
-						m.put(h, w = new DBS());
-					}
-					x.linkAlphabet(w, a, lh);
+					x.linkAlphabet(z, r, lcb + g + lce);
+					x.linkEnd(z, g);
+				}
+			}
+		}
+	}
+
+	private void _linkdfaNegate(NinaEvent q,
+			DFA<Object, NFAState, Integer> d, DBS v, DBS z,
+			Object o, String lh, String lc1, String lcb, String lce,
+			boolean en) {
+		Collection<DFAState<Object, NFAState, Integer>> c;
+		Map<DFAState<Object, NFAState, Integer>, DBS> m;
+		String g;
+		DBS x;
+
+		c = DFAs.allStateRanges(d.getInitialState());
+		m = new HashMap<DFAState<Object, NFAState, Integer>, DBS>();
+		g = lc1;
+		if(o != null)  g += o.toString();
+		for(DFAState<Object, NFAState, Integer> y : c) {
+			x = _linkdfa0(d, y, m, v, lh);
+
+			// accept if the DFA is not accepted
+			if(!y.isAccepted()) {
+				if(y.equals(d.getInitialState())) {
+					x.linkAlphabet(z,
+							CharSets.complement(_regexAllAlphabets(y)),
+							q.getLanguageOption().getUnget());
+				} else {
+					x.linkAlphabet(z,
+							CharSets.complement(_regexAllAlphabets(y)),
+							lcb + g + lce);
+					x.linkEnd(z, g);
 				}
 			}
 		}
@@ -711,14 +855,15 @@ public class DFABuilder implements NinaAction {
 
 	private void _linkre(NinaEvent q, String re, DBS x, String lh,
 			String lc) {
-		NFA<Object, NFAState, Void> n;
-		DFA<Object, NFAState, Void> d;
+		NFA<Object, NFAState, Integer> n;
+		DFA<Object, NFAState, Integer> d;
 
 		try {
 			// NFA bugs
-			n = RegexpParser.parse(re, null);
+			n = RegexParser.parse(re);
 			d = ConvertedRangeDFA.convertDFA(n);
-			_linkdfa1(q, d, x, vertex, mealy, lh, lc, false);
+			_linkdfa1(q, d, x, vertex, mealy, lh, lh + lc, "", "",
+					false);
 		} catch (RegexParseException e) {
 			throw new NinaException("invalidregex");
 		}
@@ -733,12 +878,111 @@ public class DFABuilder implements NinaAction {
 		return true;
 	}
 
-	private void _linkdq(NinaEvent q, CharSequence s, DBS v) {
+	private void _findsub(Map<String, DBS> m, String s, DBS v,
+			String k) {
+		char c, C;
+		DBS a, b;
+
+		if(k.length() < s.length()) {
+			c = s.charAt(k.length());
+			C = Character.toUpperCase(c);
+			c = Character.toLowerCase(c);
+			if(c == C) {
+				a = v.edges == null ? null : v.edges.map((int)c);
+				if(a != null) {
+					_findsub(m, s, a, k + c);
+				} else {
+					m.put(k, v);
+				}
+			} else {
+				a = v.edges == null ? null : v.edges.map((int)c);
+				b = v.edges == null ? null : v.edges.map((int)C);
+				if(a != null && b != null) {
+					_findsub(m, s, a, k + c);
+					_findsub(m, s, b, k + C);
+				} else if(a != null) {
+					_findsub(m, s, a, k + c);
+				} else if(b != null) {
+					_findsub(m, s, b, k + C);
+				} else {
+					m.put(k, v);
+				}
+			}
+		} else {
+			m.put(k, v);
+		}
+	}
+
+	private void _linksubdq(NinaEvent q, String s, DBS X, DBS v, DBS p,
+			String lh) {
+		Map<String, DBS> m;
+		DBS x, w, a, b;
+		boolean f, F;
+		int c, C;
+
+		m = new HashMap<String, DBS>();
+		_findsub(m, s, v, "");
+		if(m.containsKey("")) {
+			c = s.charAt(0);
+			C = Character.toUpperCase(c);
+			c = Character.toLowerCase(c);
+			_linkv(q, v, X, c, lh);
+			if(c != C) {
+				_linkv(q, v, X, C, lh);
+			}
+		} else {
+			f = F = true;
+			for(Map.Entry<String, DBS> t : m.entrySet()) {
+				x = X;  w = v;
+				for(int k = 0; k < t.getKey().length(); k++) {
+					c = s.charAt(k);
+					C = Character.toUpperCase(c);
+					c = Character.toLowerCase(c);
+					a = w.edges.map(c);
+					b = w.edges.map(C);
+					if(k == 0) {
+						f = f && c != C && a == null;
+						F = F && c != C && b == null;
+					} else if(a == null && b == null) {
+						_linkv(q, w, x, c, lh);
+						_linkv(q, w, x, C, lh);
+					} else if(a == null) {
+						_linkv(q, w, x, c, lh);
+					} else if(b == null) {
+						_linkv(q, w, x, C, lh);
+					}
+
+					w = w.edges.map((int)t.getKey().charAt(k));
+					if(k < s.length() - 1) {
+						x = x.edges.map((int)s.charAt(k + 1));
+					}
+				}
+
+				if(t.getKey().length() < s.length()) {
+					c = s.charAt(t.getKey().length());
+					C = Character.toUpperCase(c);
+					c = Character.toLowerCase(c);
+					_linkv(q, w, x, c, lh);
+					_linkv(q, w, x, C, lh);
+				} else {
+					w.linkOthers(p, q.getLanguageOption().getUnget());
+					w.linkEnd(p, null);
+				}
+			}
+			c = s.charAt(0);
+			C = Character.toUpperCase(c);
+			c = Character.toLowerCase(c);
+			if(f)  _linkv(q, v, X, c, lh);
+			if(F)  _linkv(q, v, X, C, lh);
+		}
+	}
+
+	private void _linkdq(NinaEvent q, CharSequence s, DBS v, int f) {
 		List<DoubleQuoteTokenizer.Elems> b;
-		String g, lh, lm, lc, lu;
+		String g, h, lh, lm, lc, lu;
+		DBS x, X, a, A;
+		int k, c, C;
 		boolean z;
-		int k, c;
-		DBS x, a;
 		Range r;
 
 		lh = q.getLanguageOption().getLookahead();
@@ -747,16 +991,52 @@ public class DFABuilder implements NinaAction {
 		lu = q.getLanguageOption().getUnget();
 		b  = DoubleQuoteTokenizer.parse(s.toString());
 		for(DoubleQuoteTokenizer.Elems p : b) {
-			x = v;
+			x = X = v;
 			s = p.getString();
 			for(k = 0; k < s.length() - 1; k++) {
-				c = s.charAt(k);
-				if(x.edges == null) {
-					x = _linkc(q, x, c, lh);
-				} else if((a = x.edges.map(c)) != null) {
-					x = a;
+				if(f == 'I') {
+					c = s.charAt(k);
+					C = Character.toUpperCase(c);
+					c = Character.toLowerCase(c);
+					if(x.edges == null) {
+						x = _linkc(q, x, c, lh);
+					} else if((a = x.edges.map(c)) != null) {
+						x = a;
+					} else {
+						x = _linkc(q, x, c, lh);
+					}
+
+					if(X.edges == null) {
+						X = _linkc(q, X, C, lh);
+					} else if((A = X.edges.map(C)) != null) {
+						X = A;
+					} else {
+						X = _linkc(q, X, C, lh);
+					}
+				} else if(f == 'i') {
+					c = s.charAt(k);
+					C = Character.toUpperCase(c);
+					c = Character.toLowerCase(c);
+					a = createState(q);
+					if(k == 0) {
+						x = X = a;
+					} else if(c == C) {
+						_linkv(q, x, a, c, lh);
+						x = a;
+					} else {
+						_linkv(q, x, a, c, lh);
+						_linkv(q, x, a, C, lh);
+						x = a;
+					}
 				} else {
-					x = _linkc(q, x, c, lh);
+					c = s.charAt(k);
+					if(x.edges == null) {
+						x = _linkc(q, x, c, lh);
+					} else if((a = x.edges.map(c)) != null) {
+						x = a;
+					} else {
+						x = _linkc(q, x, c, lh);
+					}
 				}
 			}
 
@@ -781,32 +1061,77 @@ public class DFABuilder implements NinaAction {
 				r = Interval.newPoint(Integer.valueOf(s.charAt(k)));
 				if(p.isInvertLookahead())  r = CharSets.complement(r);
 				x.linkAlphabet(vertex, r, g);
-			} else if(_isword(s)) {
+			} else if(f == 'w') {
+				g = lc + lu;
+				h = lc;
+				if(mealy != null) {
+					g += mealy.toString();
+					h += mealy.toString();
+				}
 				x = _linkc(q, x, s.charAt(k), lh);
+				x.linkAlphabet(vertex, SPACES, g);
+				x.linkEnd(vertex, h);
+			} else if(_isword(s) && f != 'p') {
 				g = lc + lu;
 				if(mealy != null)  g += mealy.toString();
-				x.linkAlphabet(vertex, NOT_WORD, g);
+				if(f == 'I') {
+					c = s.charAt(k);
+					C = Character.toUpperCase(c);
+					c = Character.toLowerCase(c);
+					x = _linkc(q, x, c, lh);
+					X = _linkc(q, X, C, lh);
+					x.linkAlphabet(vertex, NOT_WORD, g);
+					X.linkAlphabet(vertex, NOT_WORD, g);
+				} else if(f == 'i') {
+					c = s.charAt(k);
+					C = Character.toUpperCase(c);
+					c = Character.toLowerCase(c);
+					a = _linkc(q, x, c, lh);
+					if(c != C) {
+						_linkv(q, x, a, C, lh);
+					}
+					a.linkAlphabet(vertex, NOT_WORD, g);
+					_linksubdq(q, s.toString(), X, v, a, lh);
+				} else {
+					x = _linkc(q, x, s.charAt(k), lh);
+					x.linkAlphabet(vertex, NOT_WORD, g);
+				}
 			} else {
 				g = lc;
 				if(mealy != null)  g += mealy.toString();
-				r = Interval.newPoint(Integer.valueOf(s.charAt(k)));
-				x.linkAlphabet(vertex, r, g);
+				if(f == 'I') {
+					c = s.charAt(k);
+					C = Character.toUpperCase(c);
+					c = Character.toLowerCase(c);
+					_linkv(q, x, vertex, c, g);
+					_linkv(q, X, vertex, C, g);
+				} else if(f == 'i') {
+					c = s.charAt(k);
+					C = Character.toUpperCase(c);
+					c = Character.toLowerCase(c);
+					_linkv(q, x, vertex, c, g);
+					_linkv(q, x, vertex, C, g);
+					_linksubdq(q, s.toString(), X, v, vertex, lh);
+				} else {
+					_linkv(q, x, vertex, s.charAt(k), g);
+				}
 			}
 		}
 	}
 
 	private void _link1(NinaEvent q, DBS v) {
 		CharSequence s;
-		int k, b, l;
+		int k, b, l, f;
 		Range r;
 		DBS x;
 
 		s = ((W)q.getEdge()).wrap;
 		b = ((W)q.getEdge()).quote;
+		f = ((W)q.getEdge()).quoteFlag;
 		l = s.length() - 1;
 		x = v;
 		if(b == '"') {
-			_linkdq(q, s, v);
+			_linkdq(q, s, v, f);
 		} else {
 			for(k = 0; k < l; k++) {
 				x = _linkc(q, x, s.charAt(k), null);
@@ -882,8 +1207,8 @@ public class DFABuilder implements NinaAction {
 			x = new DBS();
 			v.linkRecursive(x, sub);
 			x.label = l.getAddList();
-			_linkdfa1(q, (DFA<Object, NFAState, Void>)dl, x, v, mealy,
-					"", s, true);
+			_linkdfa1(q, (DFA<Object, NFAState, Integer>)dl, x, v,
+					mealy, "", s, "", "", true);
 			x.linkOthers(vertex, t);
 			x.linkEnd(vertex, mealy);
 		} else if((a = (char[])dl).length < 1) {
@@ -1113,9 +1438,167 @@ public class DFABuilder implements NinaAction {
 		svar.linkEnd(vertex, m);
 	}
 
+	private void _linkSkipSpaces(NinaEvent q, DBS v, String str) {
+		DBS x = v;
+		int k;
+
+		v.linkAlphabet(v, SPACES, null);
+		if(str.length() > 0) {
+			for(k = 0; k < str.length() - 1; k++) {
+				x = _linkc(q, x, str.charAt(k), null);
+			}
+			x.linkAlphabet(vertex,
+					IntervalsInt.newPoint(str.charAt(k)), mealy);
+			vertex.linkAlphabet(vertex, SPACES, null);
+		} else {
+			v.linkOthers(vertex, q.getLanguageOption().getUnget());
+			v.linkEnd(vertex, null);
+		}
+	}
+
+	private void _linkBackslash(NinaEvent q, DBS v, int seq) {
+		NinaLanguageOption p = q.getLanguageOption();
+		String s, ib, ab;
+		DBS x;
+
+		switch(seq) {
+		case 'X':
+			x  = createState(q);
+			ib = p.getInitBuffer(p.getSequenceBufferName());
+			ab = p.getAppendBuffer(p.getSequenceBufferName());
+			v.linkOthers(x, ib + ab);
+			_linkv(q, x, x, MARK, ab);
+			s = mealy != null ? mealy.toString() : "";
+			s = s + p.getUnget();
+			x.linkOthers(vertex, s);
+			x.linkEnd(vertex, s);
+			break;
+		case 'b':
+			s = mealy != null ? mealy.toString() : "";
+			v.linkAlphabet(vertex, WORD, p.getUnget() + s);
+			break;
+		case 'B':
+			s = mealy != null ? mealy.toString() : "";
+			v.linkAlphabet(vertex, NOT_WORD, p.getUnget() + s);
+			break;
+		default:
+			throw new NinaException("invalidsequence");
+		}
+	}
+
+	private void _linkRegex(NinaEvent q, DBS v,
+			NFAObject<Object, NFAState, Integer> n, int f) {
+		NinaLanguageOption p = q.getLanguageOption();
+		DFA<Object, NFAState, Integer> d;
+		String h, i, e;
+		DBS x;
+
+		if(f == NO_CAPTURE) {
+			d = ConvertedRangeDFA.convertDFA(n);
+			_linkdfaAssert(q, d, v, vertex, mealy, "", "", "",
+					p.getUnget(), true);
+		} else if(f == REGEX_ATOMIC) {
+			d = ConvertedRangeDFA.convertDFA(n);
+			h = p.getLookahead();
+			e = p.getLookaheadCommit();
+			_linkdfaAssert(q, d, v, vertex, mealy, h, e, "",
+					p.getUnget(), true);
+		} else if(f == REGEX_LOOKAHEAD) {
+			d = ConvertedRangeDFA.convertDFA(n);
+			h = p.getLookahead() + p.getLookaheadMarkInit();
+			e = p.getLookaheadCommit();
+			_linkdfaAssert(q, d, v, vertex, mealy, h, e,
+					p.getLookahead(), "", true);
+		} else if(f == REGEX_LOOKAHEAD_NEGATIVE) {
+			d = ConvertedRangeDFA.convertDFA(n);
+			h = p.getLookahead() + p.getLookaheadMarkInit();
+			e = p.getLookaheadCommit();
+			_linkdfaNegate(q, d, v, vertex, mealy, h, e,
+					p.getLookahead(), "", true);
+		} else {
+			d = ConvertedRangeDFA.convertDFA(n);
+			h = p.getAppendBuffer(p.getSequenceBufferName());
+			i = p.getInitBuffer(p.getSequenceBufferName());
+			x = createState(q);
+			x.accepted = v.accepted;
+			v.linkAlphabet(x, _regexFirst(d), p.getUnget() + i);
+//			v.linkEnd(x, null);
+			_linkdfaAssert(q, d, x, vertex, mealy, h, "", "",
+					p.getUnget(), true);
+		}
+	}
+
+	//
+	private void _linkFormatS(NinaEvent q, DBS v, int l, String cs) {
+		NinaLanguageOption p;
+		String b, i, u;
+		Range r;
+		DBS x, y;
+
+		r = CharSet.parseRange(cs);
+		p = q.getLanguageOption();
+		i = p.getInitBuffer(p.getSequenceBufferName());
+		b = p.getAppendBuffer(p.getSequenceBufferName());
+		u = p.getUnget();
+		x = createState(q);
+		v.linkAlphabet(x, r, i + u);
+		if(l < 0) {
+			x.linkAlphabet(x, r, b);
+			x.linkOthers(vertex, u);
+			x.linkEnd(vertex, null);
+		} else {
+			y = x;
+			for(int k = 0; k < l - 1; k++) {
+				x = createState(q);
+				y.linkAlphabet(x, r, b);
+				y.linkOthers(vertex, u);
+				y.linkEnd(vertex, null);
+				y = x;
+			}
+			y.linkAlphabet(vertex, r, b);
+			y.linkOthers(vertex, u);
+			y.linkEnd(vertex, null);
+		}
+	}
+
+	//
+	private void _linkFormatI(NinaEvent q, DBS v, int l, String f,
+			Range r, int d) {
+		NinaLanguageOption p;
+		String b, i, u, t;
+		DBS x, y;
+
+		p = q.getLanguageOption();
+		i = p.getInitBuffer(p.getSequenceBufferName());
+		b = p.getAppendBuffer(p.getSequenceBufferName());
+		u = p.getUnget();
+		t = f != null && f.equals("D") ?
+				p.getBufferToBigInteger(d) : p.getBufferToInteger(d);
+		x = createState(q);
+		v.linkAlphabet(x, r, i + u);
+		if(l < 0) {
+			x.linkAlphabet(x, r, b);
+			x.linkOthers(vertex, t + u);
+			x.linkEnd(vertex, t);
+		} else {
+			y = x;
+			for(int k = 0; k < l - 1; k++) {
+				x = createState(q);
+				y.linkAlphabet(x, r, b);
+				y.linkOthers(vertex, t + u);
+				y.linkEnd(vertex, t);
+				y = x;
+			}
+			y.linkAlphabet(vertex, r, b + t);
+			y.linkOthers(vertex, t + u);
+			y.linkEnd(vertex, t);
+		}
+	}
+
 	/* (non-Javadoc)
 	 * @see net.morilib.nina.NinaAction#link(net.morilib.nina.NinaEvent, java.lang.Object)
 	 */
+	@SuppressWarnings("unchecked")
 	@Override
 	public void link(NinaEvent q, Object ver) {
 		DBS v = vertex;
@@ -1159,9 +1642,21 @@ public class DFABuilder implements NinaAction {
 		} else if(o instanceof DO) {
 			if(((DO)o).num < 0) {
 				_linkOctBig(q, v);
-		} else {
+			} else {
 				_linkOctInt(q, v, ((DO)o).num);
 			}
+		} else if(o instanceof WS) {
+			_linkSkipSpaces(q, v, ((WS)o).str);
+		} else if(o instanceof BS) {
+			_linkBackslash(q, v, ((BS)o).seq);
+		} else if(o instanceof FS) {
+			_linkFormatS(q, v, ((FS)o).length, ((FS)o).cset);
+		} else if(o instanceof FI) {
+			_linkFormatI(q, v, ((FI)o).length, ((FI)o).flag,
+					((FI)o).range, ((FI)o).radix);
+		} else if(o instanceof NFAObject) {
+			_linkRegex(q, v, (NFAObject<Object, NFAState, Integer>)o,
+					nfaFlag);
 		} else if(o == _FLOAT) {
 			_linkFloat(q, v);
 		} else if(o == _END) {
@@ -1225,8 +1720,8 @@ public class DFABuilder implements NinaAction {
 	 */
 	@Override
 	public void setEdgeCharSequence(NinaEvent q, CharSequence o,
-			int qc) {
-		q.setEdge(new W(o, qc));
+			int qc, int qf) {
+		q.setEdge(new W(o, qc, qf));
 	}
 
 	/* (non-Javadoc)
@@ -1253,9 +1748,11 @@ public class DFABuilder implements NinaAction {
 	 * @see net.morilib.nina.NinaAction#setEdgeNFA(net.morilib.nina.NinaEvent, net.morilib.nina.NinaParser, net.morilib.automata.NFA)
 	 */
 	@Override
-	public void setEdgeNFA(NinaEvent q, NinaParser p,
-			NFA<Object, NFAState, Void> s) {
-		throw new NinaException("cannotaddnfa");
+	public void setEdgeNFA(NinaEvent q,
+			NFA<Object, NFAState, Integer> s,
+			int f) {
+		nfaFlag = f;
+		q.setEdge(s);
 	}
 
 	/* (non-Javadoc)
@@ -1504,4 +2001,46 @@ public class DFABuilder implements NinaAction {
 		}
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#setEdgeSkipSpaces(net.morilib.nina.NinaEvent, java.lang.String)
+	 */
+	@Override
+	public void setEdgeSkipSpaces(NinaEvent q, String str) {
+		q.setEdge(new WS(str));
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#setEdgeBackslash(net.morilib.nina.NinaEvent, int)
+	 */
+	@Override
+	public void setEdgeBackslash(NinaEvent q, int c) {
+		q.setEdge(new BS(c));
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#setEdgeStringSequence(net.morilib.nina.NinaEvent, int, java.lang.String)
+	 */
+	@Override
+	public void setEdgeStringSequence(NinaEvent q, int l, String cs) {
+		q.setEdge(new FS(l, cs));
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#setEdgeFormatInteger(net.morilib.nina.NinaEvent, int, java.lang.String)
+	 */
+	@Override
+	public void setEdgeFormatInteger(NinaEvent q, int l, String flg) {
+		q.setEdge(new FI(l, flg, DIGIT, 10));
+	}
+
+	@Override
+	public void setEdgeFormatHex(NinaEvent q, int l, String flg) {
+		q.setEdge(new FI(l, flg, HEXD, 16));
+	}
+
+	@Override
+	public void setEdgeFormatOct(NinaEvent q, int l, String flg) {
+		q.setEdge(new FI(l, flg, OCTD, 8));
+	}
+
 }
