@@ -619,6 +619,11 @@ then
 		__lookahead_mark = __lookaheadw_ptr;
 	}
 
+	void LOOKAHEAD_MARK_INIT()
+	{
+		__lookahead_mark = 0;
+	}
+
 EOF
 fi
 
@@ -1112,8 +1117,18 @@ EOF
 				}
 				else if(c == $nullsym)
 				{
-					if(!b)  throw new ${TOKENERROR}();
-					st = NinaAccept;  return $nullsym;
+EOF
+  [ -n "$MATCHER" ] || echo "					if(!b)  throw new ${TOKENERROR}();"
+  [ -n "$MATCHER" ] && cat << EOF
+					if(!b)
+					{
+						st = NinaFail;
+						return $nullsym;
+					}
+EOF
+  cat << EOF
+					st = NinaAccept;
+					return $nullsym;
 				}
 				else
 				{
@@ -1234,7 +1249,10 @@ EOF
 
 							if(__slen-- <= 1)
 							{
-								throw new TokenException();
+EOF
+  [ -n "$MATCHER" ] || echo "								throw new ${TOKENERROR}();"
+  [ -n "$MATCHER" ] && echo '								return false;'
+  cat << EOF
 							}
 						}
 						skip = true;
@@ -1279,7 +1297,9 @@ EOF
 				}
 				b = true;
 			}
-			if(!((bool)b))  throw new ${TOKENERROR}();
+EOF
+  [ -n "$MATCHER" ] || echo "			if(!((bool)b))  throw new ${TOKENERROR}();"
+  cat << EOF
 			return (bool)b;
 		}
 		finally
